@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2008 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -17,45 +17,43 @@
 #ifndef BZF_COMMON_H
 #define	BZF_COMMON_H
 
-// this should always be the very FIRST header
-#ifdef _DEVCPP //the Dev-C++ build is acting very stubborn; this is (hopefully) -temporary-
+/* this should always be the very FIRST header */
+
+#ifdef _DEVCPP /* the Dev-C++ build is acting very stubborn; this is (hopefully) -temporary- */
 # include_next "config.h"
 #else
 # include "config.h"
 #endif
 
 #ifdef _WIN32
-#include "win32.h"
-#define _FD_SET(fd, set) FD_SET((unsigned int)fd, set)
-#else
-#define _FD_SET(fd, set) FD_SET(fd, set)
+#  undef NOMINMAX
+#  define NOMINMAX 1
+#  include "win32.h"
 #endif
 
 #include <stdio.h>
-#include <stdlib.h> //needed for bzfrand
+#include <stdlib.h> /* needed for bzfrand */
 #include <math.h>
+#ifdef __cplusplus
+#  include <cmath>
+#endif
+
 
 extern int debugLevel;
-// Like verbose debug messages? level 0 for development only
-#define DEBUG0 formatDebug
-#define DEBUG1 if (debugLevel >= 1) formatDebug
-#define DEBUG2 if (debugLevel >= 2) formatDebug
-#define DEBUG3 if (debugLevel >= 3) formatDebug
-#define DEBUG4 if (debugLevel >= 4) formatDebug
 
 /* near zero by some epsilon convenience define since relying on
 * the floating point unit for proper equivalence is not safe
 */
 #define NEAR_ZERO(_value,_epsilon)  ( ((_value) > -_epsilon) && ((_value) < _epsilon) )
 
-// seven places of precision is pretty safe, so something less precise
-#if defined(FLT_EPSILON)
+/* seven places of precision is pretty safe, so something less precise */
+#ifdef FLT_EPSILON
 #  define ZERO_TOLERANCE FLT_EPSILON
 #else
 #  define ZERO_TOLERANCE 1.0e-06f
 #endif
 
-// Might we be BSDish? sys/param.h has BSD defined if so
+/* Might we be BSDish? sys/param.h has BSD defined if so */
 #ifdef HAVE_SYS_PARAM_H
 #  include <sys/param.h>
 #endif
@@ -66,7 +64,7 @@ extern int debugLevel;
 #ifdef HAVE__STRNICMP
 #  define strncasecmp _strnicmp
 #endif
-#if !defined(HAVE_VSNPRINTF)
+#ifndef HAVE_VSNPRINTF
 #  ifdef HAVE__VSNPRINTF
 #    define vsnprintf _vsnprintf
 #  else
@@ -74,7 +72,7 @@ extern int debugLevel;
 #  endif
 #endif
 
-// some platforms don't have float versions of the math library
+/* some platforms don't have float versions of the math library */
 #ifndef HAVE_ASINF
 #  define	asinf		(float)asin
 #endif
@@ -105,6 +103,9 @@ extern int debugLevel;
 #ifndef HAVE_LOGF
 #  define	logf		(float)log
 #endif
+#ifndef HAVE_LOG10F
+#  define	log10f		(float)log10
+#endif
 #ifndef HAVE_POWF
 #  define	powf		(float)pow
 #endif
@@ -118,12 +119,11 @@ extern int debugLevel;
 #  define	tanf		(float)tan
 #endif
 
-// random number stuff
+
+/* random number stuff */
 #define bzfrand()	((double)rand() / ((double)RAND_MAX + 1.0))
 #define bzfsrand(_s)	srand(_s)
 
-#if !defined(_WIN32)
-
 #ifndef __BEOS__
 #  ifdef HAVE_VALUES_H
 #    include <values.h>
@@ -136,55 +136,55 @@ extern int debugLevel;
 #  define MAXLONG LONG_MAX
 #endif /* __BEOS__ */
 
-#include <sys/types.h>
+#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
+
+/* need some integer types */
+#ifdef HAVE_INTTYPES_H
+#  include <inttypes.h>
+#endif
 
 #ifdef HAVE_STDINT_H
 #  include <stdint.h>
-#endif
-
-#if defined(__linux) || (defined(__sgi) && !defined(__INTTYPES_MAJOR))
+#else
+#  if defined(__linux) || (defined(__sgi) && !defined(__INTTYPES_MAJOR))
 typedef u_int16_t	uint16_t;
 typedef u_int32_t	uint32_t;
-#endif
-
-#if defined(sun)
+#  endif
+#  if defined(sun)
 typedef signed short	int16_t;
 typedef ushort_t	uint16_t;
 typedef signed int	int32_t;
 typedef uint_t		uint32_t;
-#endif
-
-#endif
-
+#  endif
 typedef unsigned char	uint8_t;
+#endif
 
-#if defined( __BEOS__ )
 
-// missing constants
+/* missing constants */
 
-#  ifndef MAXFLOAT
-#    define	MAXFLOAT	3.402823466e+38f
-#  endif
+#ifndef MAXFLOAT
+#  define	MAXFLOAT	3.402823466e+38f
+#endif
 
-#  ifndef M_PI
-#    define	M_PI		  3.14159265358979323846f
-#  endif
+#ifndef M_PI
+#  define	M_PI		3.14159265358979323846f
+#endif
 
-#  ifndef M_SQRT1_2
-#    define	M_SQRT1_2	0.70710678118654752440f
-#  endif
+#ifndef M_SQRT1_2
+#  define	M_SQRT1_2	0.70710678118654752440f
+#endif
 
-// need some integer types
-#  include <inttypes.h>
 
+#ifdef __BEOS__
 #  ifndef setenv
 #    define setenv(a,b,c)
 #  endif
-
 #  ifndef putenv
 #    define putenv(a)
 #  endif
-#endif /* defined( __BEOS__ ) */
+#endif /* __BEOS__ */
 
 #ifdef countof
 #  undef countof
@@ -192,13 +192,92 @@ typedef unsigned char	uint8_t;
 #define countof(__x)   (sizeof(__x) / sizeof(__x[0]))
 
 
-#endif // BZF_COMMON_H
+#ifdef HAVE_STD__ISNAN
+#  ifdef isnan
+#    undef isnan
+#  endif
+#  define isnan std::isnan
+#elif defined(HAVE__ISNAN)
+#  ifdef isnan
+#    undef isnan
+#  endif
+#  define isnan _isnan
+#else
+#  ifndef HAVE_ISNAN
+#    ifdef __cplusplus
+#      ifdef isnan
+#        undef isnan
+#      endif
+       template<typename Tp>
+       inline int isnan(Tp f)
+       {
+         return (f!=f);
+       }
+#    else
+#      define isnan(f) ((f) != (f))
+#    endif /* __cplusplus */
+#  endif /* HAVE_ISNAN */
+#endif /* HAVE_STD__ISNAN */
 
 
-// Local Variables: ***
-// mode: C++ ***
-// tab-width: 8 ***
-// c-basic-offset: 2 ***
-// indent-tabs-mode: t ***
-// End: ***
-// ex: shiftwidth=2 tabstop=8
+#ifndef HAVE_STD__MAX
+#  ifdef __cplusplus
+#    ifdef max
+#      undef max
+#    endif
+     namespace std
+     {
+       template<typename comparable>
+       inline const comparable& max(const comparable& a, const comparable& b)
+       {
+	 return  a < b ? b : a;
+       }
+     }
+#  else
+#    ifdef max
+#      undef max
+#    endif
+#    define max(a,b) a < b ? b : a
+#  endif /* __cplusplus */
+#endif /* HAVE_STD__MAX */
+
+#ifndef HAVE_STD__MIN
+#  ifdef __cpluscplus
+#    ifdef min
+#      undef min
+#    endif
+     namespace std
+     {
+       template<typename comparable>
+       inline const comparable& min(const comparable& a, const comparable& b)
+       {
+	 return b < a ? b : a;
+       }
+     }
+#  else
+#    ifdef min
+#      undef min
+#    endif
+#    define min(a,b) b < a ? b : a
+#  endif /* __cplusplus */
+#endif /* HAVE_STD_MIN */
+
+#ifdef BUILD_REGEX
+#  include "bzregex.h"
+#elif defined(HAVE_REGEX_H)
+#  include <regex.h>
+#else
+#  define regex_t void
+#endif  /* BUILD_REGEX */
+
+#endif /* BZF_COMMON_H */
+
+
+/* Local Variables: ***
+ * mode: C++ ***
+ * tab-width: 8 ***
+ * c-basic-offset: 2 ***
+ * indent-tabs-mode: t ***
+ * End: ***
+ * ex: shiftwidth=2 tabstop=8
+ */
