@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993 - 2008 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef	__PLAYER_H__
@@ -20,7 +20,9 @@
 
 /* common interface headers */
 #include "global.h"
+#ifndef BUILDING_BZADMIN
 #include "bzfgl.h"
+#endif
 #include "TimeKeeper.h"
 #include "Address.h"
 #include "Flag.h"
@@ -73,6 +75,7 @@ public:
   float		getAngle() const;
   const float*	getForward() const;
   const float*	getVelocity() const;
+  
   float		getAngularVelocity() const;
   int		getPhysicsDriver() const;
   int		getDeathPhysicsDriver() const;
@@ -82,8 +85,20 @@ public:
   short		getWins() const;
   short		getLosses() const;
   short		getTeamKills() const;
+  float		getTKRatio() const;
+  float		getNormalizedScore() const;
+  float		getLocalNormalizedScore() const;
   short		getScore() const;
   const float*	getDimensions() const;
+
+  const float*	getApparentVelocity() const;
+
+#ifndef BUILDING_BZADMIN
+  inline const float*	getColor() const
+  {
+	  return color;
+  }
+#endif
   short		getRabbitScore() const;
   short		getLocalWins() const;
   short		getLocalLosses() const;
@@ -102,6 +117,9 @@ public:
 			   bool inCockpit, bool seerView,
 			   bool showTreads, bool showIDL);
 
+  bool		getIpAddress(Address&);
+  void		setIpAddress(const Address& addr);
+
   virtual void	addShots(SceneDatabase*, bool colorblind) const;
   void		setLandingSpeed(float velocity);
   void		spawnEffect();
@@ -130,6 +148,9 @@ public:
   bool		hasPlayerList() const;
   void		setPlayerList(bool = true);
 
+  bool		getPausedMessageState ( void ) const;
+  void		setPausedMessageState ( bool set ) { pauseMessageState = set;}
+
   bool		validTeamTarget(const Player *possibleTarget) const;
 
   // returns true iff dead reckoning is too different from the
@@ -169,6 +190,13 @@ public:
 
   void setZpos (float z);
 
+  float getMaxSpeed ( void ) const;
+
+  void forceReload(float time);
+
+  int reportedHits;
+  int computedHits;
+  std::map<int,bool>	hitMap;
 protected:
   void	  clearRemoteSounds();
   void	  addRemoteSound(int sound);
@@ -177,6 +205,10 @@ protected:
   // shot statistics
   ShotStatistics	shotStatistics;
   const Obstacle* lastObstacle; // last obstacle touched
+  TimeKeeper	jamTime;
+
+  // pause message
+  bool pauseMessageState;
 
 private:
   // return true if the shot had to be terminated or false if it
@@ -195,6 +227,8 @@ private:
   void updateJumpJets(float dt);
   void updateTrackMarks();
   bool hitObstacleResizing();
+
+
 private:
   // data not communicated with other players
   bool			notResponding;
@@ -206,13 +240,17 @@ private:
   bool			registered;
   bool			verified;
   bool			playerList;
+  Address		ipAddr;
+  bool			haveIpAddr;
 
   // data use for drawing
   TankSceneNode*	tankNode;
   TankIDLSceneNode*	tankIDLNode;
   SphereSceneNode*	pausedSphere;
+#ifndef BUILDING_BZADMIN
   GLfloat		color[4];
   GLfloat		teleAlpha;
+#endif
   std::string		userTexture;
   static int		tankTexture;
   static int		tankOverideTexture;
@@ -236,6 +274,7 @@ private:
   float			alpha;			// current tank translucency
   float			alphaRate;		// current tank translucency
   float			alphaTarget;		// current tank translucency
+  TimeKeeper		spawnTime;		// time I started spawning
   TimeKeeper		explodeTime;		// time I started exploding
   TimeKeeper		teleportTime;		// time I started teleporting
   short			fromTeleporter;		// teleporter I entered
@@ -263,8 +302,11 @@ private:
   float			relativeSpeed;		// relative speed
   float			relativeAngVel;		// relative angular velocity
 
+  float			apparentVelocity[3];	// velocity of tank as derived from it's last positional update
+
   // dead reckoning stuff
   TimeKeeper inputTime;		// time of input
+  float inputTimestamp;         // input timestamp of sender
   int	inputStatus;		// tank status
   float	inputPos[3];		// tank position
   float	inputVel[3];		// tank velocity
@@ -369,6 +411,11 @@ inline const float*	Player::getVelocity() const
   return state.velocity;
 }
 
+inline const float*	Player::getApparentVelocity() const
+{
+  return apparentVelocity;
+}
+
 inline float		Player::getAngularVelocity() const
 {
   return state.angVel;
@@ -454,6 +501,11 @@ inline bool		Player::isPaused() const
   return (state.status & short(PlayerState::Paused)) != 0;
 }
 
+inline bool Player::getPausedMessageState ( void ) const
+{ 
+	return pauseMessageState;
+}
+
 inline bool		Player::isAutoPilot() const
 {
   return autoPilot;
